import { isString, isNumeric, isFunction, includes } from '@ntks/toolbox';
import { isWidgetDependency, getWidget, } from '../vendors/organik';
import { BuiltInDataType } from '../types/data-type';
import { cacheDynamicEnumOptions, getCachedEnumOptions } from '../utils/input';
function resolveWidgetCtor(moduleContext, widget, widgetNameFromRenderTypeResolver) {
    if (widget) {
        if (isString(widget)) {
            return isWidgetDependency(moduleContext.getModuleName(), widget)
                ? moduleContext.getComponents()[widget]
                : undefined;
        }
        return widget;
    }
    return getWidget(widgetNameFromRenderTypeResolver());
}
function renderFormFieldNodes(descriptors, arrangement, renderChild, renderRow) {
    const formFieldNodes = [];
    const rows = (arrangement || '').split('|');
    let needLayout = false;
    if (rows.length > 0) {
        const availableRows = [];
        rows.forEach(row => {
            if (isNumeric(row) && Number(row) > 0) {
                availableRows.push(row);
            }
        });
        needLayout = availableRows.length === rows.length;
    }
    if (needLayout) {
        const remainedFilters = descriptors.filter(({ hidden }) => hidden !== true);
        do {
            formFieldNodes.push(renderRow(remainedFilters.splice(0, rows.shift() * 1), remainedFilters.length));
        } while (remainedFilters.length > 0 && rows.length > 0);
        if (remainedFilters.length > 0) {
            formFieldNodes.push(renderRow(remainedFilters, -1));
        }
    }
    else {
        descriptors.forEach(descriptor => {
            if (!descriptor.hidden) {
                formFieldNodes.push(renderChild(descriptor));
            }
        });
    }
    return formFieldNodes;
}
function resolvePlaceholder(fieldOrFilter, behaviorFromConfig, behaviorFromTheme) {
    let defaultPlaceholder = '';
    if (fieldOrFilter.dataType) {
        defaultPlaceholder = `${includes(fieldOrFilter.dataType, [
            BuiltInDataType.String,
            BuiltInDataType.Text,
            BuiltInDataType.Integer,
            BuiltInDataType.Float,
        ])
            ? '请输入'
            : '请选择'}${fieldOrFilter.label || ''}`;
    }
    const showHintAsPlaceholder = behaviorFromConfig === undefined ? behaviorFromTheme : behaviorFromConfig;
    const placeholder = fieldOrFilter.placeholder || defaultPlaceholder;
    return showHintAsPlaceholder ? fieldOrFilter.hint || placeholder : placeholder;
}
function resolveEnumOptions(context, fieldOrFilter, callback) {
    const { options } = fieldOrFilter;
    if (isFunction(options)) {
        const moduleName = context.getModuleContext().getModuleName();
        const cachedOptions = getCachedEnumOptions(moduleName, fieldOrFilter);
        if (cachedOptions) {
            callback(cachedOptions);
        }
        else {
            options().then(({ success, data }) => {
                if (success) {
                    cacheDynamicEnumOptions(moduleName, fieldOrFilter, data);
                    callback(data);
                }
            });
        }
    }
    else {
        callback(options || []); // eslint-disable-line node/no-callback-literal
    }
}
export { resolveWidgetCtor, renderFormFieldNodes, resolvePlaceholder, resolveEnumOptions };
